@@ -17,7 +17,7 @@ use Helios;
 use Helios::Error;
 use Helios::LogEntry::Levels qw(:all);
 
-our $VERSION = '2.40';
+our $VERSION = '2.41';
 
 =head1 NAME
 
@@ -226,7 +226,8 @@ our %DEFAULTS = (
     PID_PATH => File::Spec->catfile(File::Spec->rootdir, 'var', 'run', 'helios'),
     MASTER_LAUNCH_INTERVAL => 1,
     ZERO_LAUNCH_INTERVAL => 10,
-    ZERO_SLEEP_INTERVAL => 30
+    ZERO_SLEEP_INTERVAL => 10,
+    REGISTRATION_INTERVAL => 60
 );
 our $CLEAN_SHUTDOWN = 1;				# used to determine if we should remove the PID file or not (at least for now)
 
@@ -237,7 +238,7 @@ our $MASTER_LAUNCH_INTERVAL;
 our $ZERO_LAUNCH_INTERVAL;
 
 our $START_TIME = time();				# used to measure uptime from the registry table
-our $REGISTRATION_INTERVAL = 300;		# used to periodically register daemon in database
+our $REGISTRATION_INTERVAL = 60;		# used to periodically register daemon in database
 our $REGISTRATION_LAST = 0;
 
 our $PID_FILE;							# globally accessible PID file location
@@ -248,6 +249,10 @@ our $ZERO_SLEEP_INTERVAL;				# to reduce needless checking of the database
 our $ZERO_SLEEP_LOG_INTERVAL = 3600;	# to reduce needless log msgs in log_tb
 our $ZERO_SLEEP_LOG_LAST = 0;
 
+our $WORKER_PROCESS = 0;				# used to indicate process has become a worker process
+										# this is used in addition to getppid() to prevent workers
+										# from becoming daemons in case of database instability
+
 # print help if asked
 if ( !defined($CLASS) || ($CLASS eq '--help') || ($CLASS eq '-h') ) {
 	require Pod::Usage;
@@ -269,7 +274,11 @@ unless ( $worker_class->can('new') ) {
         eval "require $worker_class";
         die $@ if $@;
 }
-print $worker_class, ' ', $worker_class->VERSION," loaded.\n";
+if ( defined($worker_class->VERSION) ) {
+	print $worker_class, ' ', $worker_class->VERSION," loaded.\n";
+} else{
+	print $worker_class," loaded.\n";
+}
 if ($DEBUG_MODE) { print "Debug Mode enabled.\n"; }
 
 # instantiate a worker to access the necessary settings
@@ -283,8 +292,7 @@ if ( defined($HELIOS_INI) ) {
 $worker->setJobType($worker_class);
 $worker->debug($DEBUG_MODE);
 eval {
-        $worker->getConfigFromIni();
-        $worker->getConfigFromDb();
+	$worker->prep();
 };
 if ($@) {
         die("FAILED to get configuration: $@");
@@ -314,6 +322,11 @@ if ( defined($params->{zero_sleep_interval}) ) {
 } else {
 	$ZERO_SLEEP_INTERVAL = $DEFAULTS{ZERO_SLEEP_INTERVAL};
 }
+if ( defined($params->{registration_interval}) ) {
+	$REGISTRATION_INTERVAL = $params->{registration_interval};
+} else {
+	$REGISTRATION_INTERVAL = $DEFAULTS{REGISTRATION_INTERVAL};
+}
 
 if ($DEBUG_MODE) { 
 	print "MASTER LAUNCH INTERVAL: $MASTER_LAUNCH_INTERVAL\n"; 
@@ -448,7 +461,7 @@ MAIN_LOOP:{
 					}
 					if ($DEBUG_MODE) { print "$0 $worker_class HOLDING\n"; }
 					sleep 60; 
-					# after the first cycle through HOLD, the workers had BETTER be dead  #[]?
+					# after the first cycle through HOLD, the workers had BETTER be dead 
 					if ( defined($params->{WORKER_MAX_TTL}) && $params->{WORKER_MAX_TTL} > 0 
 					       && scalar(keys %workers) ) {
 					    double_clutch();
@@ -472,7 +485,7 @@ MAIN_LOOP:{
 					$worker->logMsg(LOG_DEBUG, $waiting_jobs . " waiting " . $worker_class . " jobs.");
 				}
 				unless ( $waiting_jobs ) {
-						$times_sleeping++;	  #[]
+						$times_sleeping++;	  
 						# only log the "0 workers running, 0 workers in queue.  SLEEPING" message every $ZERO_SLEEP_LOG_INTERVAL seconds
 						# (necessary to prevent overwhelming database logging with messages we don't care about)
 						if ( ($running_workers == 0) && (($ZERO_SLEEP_LOG_LAST + $ZERO_SLEEP_LOG_INTERVAL ) < time()) ) {
@@ -507,6 +520,7 @@ MAIN_LOOP:{
 										sleep 1;
 								} elsif (defined $pid) { # $pid is zero here if defined
 										# I'm the child!
+										$WORKER_PROCESS = 1;
 										launch_worker();
 							} elsif ($! == EAGAIN) {
 								# EAGAIN is the supposedly recoverable fork error
@@ -550,6 +564,11 @@ MAIN_LOOP:{
 		
 	} otherwise {
 		my $e = shift;
+		# if we're a worker process and we ended up here
+		# it's a fluke caused by the database instability
+		# we have to bail out and hope the service daemon survives
+		# since the default logger will also be affected by this error, we can't log anything either :(
+		if ( (getppid() > 1) || ($WORKER_PROCESS == 1) ) { exit(42); }
 		if ($DEBUG_MODE) { 
 			print "EXCEPTION THROWN: ",$e->text,"\n"; 
 			print "ATTEMPTING TO RECONNECT...\n";
@@ -592,7 +611,6 @@ if ($CLEAN_SHUTDOWN) {
 	# if we're cleanly shutting down (we were told to via helios_params_tb)
 	# unregister from Helios database
 	# remove our PID
-#[]	clean_shutdown();
     terminator();
 } else {
 	# if we've had a db error, this will throw an exception when it tries to log to the database
@@ -600,7 +618,6 @@ if ($CLEAN_SHUTDOWN) {
 	$worker->logMsg(LOG_ERR,"$0 $worker_class HALTED on error: ".$worker->errstr);
 }
 
-#[] $worker->logMsg(LOG_NOTICE, "$0 $worker_class HALTED");
 if ($DEBUG_MODE) { print "$0 $worker_class HALTED!\n"; }
 
 exit();
