@@ -1,15 +1,8 @@
 package Helios::Error::DatabaseError;
 
-use 5.008;
-use strict;
-use warnings;
 use base qw(Helios::Error::BaseError);
 
-our $VERSION = '2.811_4150';
-
-# 2011-12-18:  Changed base class from Error::Simple to Helios::Error::BaseError.
-# [2014-10-10] [LH]: Added 5.008, strict, and warnings pragmas for Kwalitee 
-# ratings.
+our $VERSION = '2.40';
 
 1;
 __END__;
