@@ -1,16 +1,16 @@
 package Helios::Error::Warning;
 
-use base qw(Error::Simple);
+use base qw(Helios::Error::BaseError);
 
-our $VERSION = '1.20';
+our $VERSION = '2.40';
 
 1;
 __END__;
 
 =head1 NAME
 
-Helios::Error::Warning -  exception class for Helios job processing system; implies the job was
-successful but there was something that should be logged as an error
+Helios::Error::Warning -  exception class for Helios indicating a job was
+successful but it encountered errors during processing
 
 =head1 SYNOPSIS
 
